local CUSTOMEMERALD_VERSION = 1

if not juggy then rawset(_G, "juggy", {}) end -- Create the juggy table if it doesn't exist.
-- Quit early if the last version loaded is higher than this local file.
if juggy.CUSTOMEMERALD_VERSION and juggy.CUSTOMEMERALD_VERSION > CUSTOMEMERALD_VERSION then return end
juggy.CUSTOMEMERALD_VERSION = CUSTOMEMERALD_VERSION -- This is the newest version now.

local usingCustomEmerald = false
local emeraldSprite = nil
local emeraldColor = nil
local noEmeraldOverlay = false

freeslot("S_CUSTOMEMERALD")

local function isPositiveResult(value)
	local num = tonumber(value) or 0

	return string.lower(value) == "true" or
		string.lower(value) == "on" or
		string.lower(value) == "yes" or
		num > 0
end

states[S_CUSTOMEMERALD] =
{
	sprite = SPR_EMRC,
	frame = FF_FULLBRIGHT|FF_ANIMATE|A,
	tics = -1,
	var1 = 1,
	var2 = 1
}

addHook("MobjThinker", function(mobj)
	if juggy.CUSTOMEMERALD_VERSION > CUSTOMEMERALD_VERSION then return end
	if leveltime ~= 3 then return end
	if not usingCustomEmerald then return end
	if emeraldSprite then mobj.state = S_CUSTOMEMERALD end
	mobj.sprite = emeraldSprite or $
	mobj.color = emeraldColor or $	
end, MT_EMERALD)

addHook("MobjThinker", function(mobj)
	if juggy.CUSTOMEMERALD_VERSION > CUSTOMEMERALD_VERSION then return end
	if leveltime ~= 3 then return end
	if not usingCustomEmerald then return end
	if noEmeraldOverlay and mobj.target and mobj.target.valid and mobj.target.state == S_CUSTOMEMERALD then
		P_RemoveMobj(mobj)
	end
end, MT_OVERLAY)

addHook("MapChange", function(mapnum)
	if juggy.CUSTOMEMERALD_VERSION > CUSTOMEMERALD_VERSION then return end
	local sprite = mapheaderinfo[mapnum].customemeraldsprite
	local color = mapheaderinfo[mapnum].customemeraldcolor
	local noOverlay = mapheaderinfo[mapnum].noemeraldoverlay or 0
	
	usingCustomEmerald = sprite ~= nil or color ~= nil

	if not usingCustomEmerald then
		emeraldSprite = nil
		emeraldColor = nil
		return
	end
	
	if sprite then
		local prefix = string.sub(sprite, 1, 4)
		if prefix == "SPR_" then
			emeraldSprite = _G[string.upper(sprite)]
		else
			emeraldSprite = _G["SPR_" + string.upper(sprite)]
		end
	else
		emeraldSprite = nil
	end	

	
	if color then
		emeraldColor = tonumber(color)
		if not emeraldColor then
			local prefix = string.sub(color, 1, 10)
			if prefix == "SKINCOLOR_" then
				emeraldColor = _G[string.upper(color)]
			else
				emeraldColor = _G["SKINCOLOR_" + string.upper(color)]
			end
		end
	else
		emeraldColor = nil
	end
	
	noEmeraldOverlay = isPositiveResult(noOverlay)
end)

addHook("NetVars", function(network)
	if juggy.CUSTOMEMERALD_VERSION > CUSTOMEMERALD_VERSION then return end
	usingCustomEmerald = network($)
	if usingCustomEmerald then
		emeraldSprite = network($)
		emeraldColor = network($)
	end
end)